
//#include <DueTimer.h>
//for multiplexed rotary encoders
#ifndef MultiKnob_h
#define MultiKnob_h

class MultiKnob {
  public:
    MultiKnob(int count, int pina, int pinb, int pinc, int pind, int datapin);
    void callibrate(byte mss);
    void start();
    void clear();
    void clearstatusbit(int n, int b);
    long getpos(byte n);
    void setpos(byte n, byte v);
    void setbound(int x, int y, int t);
    int moved(byte n);
    byte update();

#ifdef KNOBS_USE_EXTERNAL
    bool knobs[8][4];
#endif
    int knobpos[8];
  private:
    int pa, pb, pc, pd, data;
#ifndef KNOBS_USE_EXTERNAL
    bool knobs[8][4];
#endif

    int knobposold[8];
    byte knobstate[8];
    int knobsens[8];
    int pinold[16];
    byte knobBits[8];
    byte dir[8];
    byte numknobs;
    long debounce;
    int debouncetime;
    int sample;
    int minv = 0, maxv = 255, turns = 2;
};

void MultiKnob::setpos(byte n, byte v) {
  if (n == 0) return;
  //n = n  % numknobs;
  int dta;
  Serial.print("SET KNOB ");
   Serial.print(v);
  v = constrain(v, minv, maxv);
  dta = map(dta, minv, maxv, 0, 255);
  Serial.print(" ");
  Serial.println(dta);
  knobpos[n - 1] = dta;
}

void MultiKnob::clearstatusbit(int n, int b) {
  if (n == 0) return;
  // n = n  % numknobs;
  bitWrite(knobBits[n - 1], b, 0);
}

void MultiKnob::callibrate(byte mss) {
  //  long st = millis();
  //    while (millis() - st < mss){
  //int sensorValue = analogRead(data);
  //if (sensorValue > knobsens[padn]) knobsens[padn] = sensorValue + 1;
  //}
}

void MultiKnob::setbound(int x, int y, int t)
{
  minv = x;
  maxv = y;
  turns = t;
}

void MultiKnob::clear() {
  for (int x = 0; x < numknobs; x++) {
    int dummy = moved(x + 1);
  }
}

int MultiKnob::moved(byte n) {
  if (n == 0) return 0;
  //n = n  % numknobs;
  bool nb = bitRead (knobBits[n - 1], 0);
  return nb;
}

byte MultiKnob:: update(void) {
  byte updated = 0;
  byte count = 0;
  //readall knobs
  //if (sample < 15){
#ifndef KNOBS_USE_EXTERNAL
  for (int padnx = 0; padnx < (numknobs * 2) ; padnx++) {
    digitalWrite(pa, (padnx >> 0 ) & 1);
    digitalWrite(pb, (padnx >> 1 ) & 1);
    digitalWrite(pc, (padnx >> 2 ) & 1);
    digitalWrite(pd, (padnx >> 3 ) & 1);
    //delayMicroseconds(2000);
    int sensorValue = analogRead(data) ;
    int tst = abs(sensorValue - pinold[padnx]);

    if (tst > 10)  pinold[padnx] = sensorValue;
    //if (padn == 7 - 1 || padn == 15 - 1)  Serial.println(sensorValue);// > 400);
    knobs[padnx / 2][count] = pinold[padnx] > 485; //sensorValue; //> 200 & sensorValue < 700 ;
    //knobs[padnx /2][count] = sensorValue;
    count = count ^ 1;
  }
#endif

  for (int padnx = 0; padnx < numknobs; padnx++) {
    int r = 0;
    uint8_t s = knobstate[padnx] & 3;

    if (knobs[padnx][0]) s |= 4;
    if (knobs[padnx][1]) s |= 8;
    switch (s) {
      case 0: case 5: case 10: case 15:
        break;
      case 1: case 7: case 8: case 14:
        //if (knobpos[padnx] < 255)
        knobpos[padnx]++; break;
      case 2: case 4: case 11: case 13:
        //if (knobpos[padnx] > 0)
        knobpos[padnx]--; break;
      case 3: case 12:
        knobpos[padnx] += 2; break;
      default:
        knobpos[padnx] -= 2; break;
    }



    if (knobposold[padnx] != knobpos[padnx]) {
      int dirx = 0;
      bitWrite (knobBits[padnx], 0, 1);
      if (knobposold[padnx] > knobpos[padnx]) dirx = -1;
      if (knobposold[padnx] < knobpos[padnx]) dirx = 1;
      knobpos[padnx] = constrain(knobpos[padnx], 0, 255);

      knobposold[padnx] = knobpos[padnx];
      dir[padnx] = dirx;
      updated++;
    }
    knobstate[padnx] = (s >> 2);
  }
  return updated;
}

void MultiKnob::start(void) {
  pinMode(pa, OUTPUT);
  pinMode(pb, OUTPUT);
  pinMode(pc, OUTPUT);
  pinMode(pd, OUTPUT);
  pinMode(data, INPUT);
  long temp = millis();
  while (millis() - temp < 100) {
    update();
  }
  debounce = millis();
  for (byte x = 0; x < numknobs; x++) {
    moved(x);
    knobpos[x] = 0;
  }
}

MultiKnob::MultiKnob(int count, int pina, int pinb, int pinc, int pind, int datapin) {
  numknobs = count;
  pa = pina;
  pb = pinb;
  pc = pinc;
  pd = pind;
  data = datapin;
}

long MultiKnob::getpos(byte n)
{
  if (n == 0) return 0;
  //n = n  % numknobs;
  int dta = knobpos[n - 1] * turns;
  dta = map(dta, 0, 255, minv, maxv);
  dta = constrain(dta, minv, maxv);
  return dta;
}

#endif


/*

  MultiKnob knob(8,8,9,10,11,A1);

  MultiKnob knob(8,pinA,pinB,pinC,pinD,A1);//quick and dirty encoder

  void setup() {
  // put your setup code here, to run once:
  Serial.begin(9600);
  }


  void loop() {
  // put your main code here, to run repeatedly:

  //delayMicroseconds(2000);
  //update();
  //Serial.print(state);
  knob.update();
  int k = knob.moved(1);
  if (k) {Serial.print(knob.getpos(1));Serial.println(k);}//knob.getpos(1));

   //Serial.print(knob.knobs[0][0]);
   //Serial.print(" ");
   // Serial.println(knob.knobs[0][1]);

  //Serial.print(knobs[0][0]);
  // Serial.print(" ");
  //Serial.print(knobs[0][1]);
  //Serial.print(sensorValue);
  // Serial.println("");
  //delay(100);
  //Serial.println("----------------------------");
  }
*/
